# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: release-1.35
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from kubernetes.client.configuration import Configuration


class V1beta2AllocationResult(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'allocation_timestamp': 'datetime',
        'devices': 'V1beta2DeviceAllocationResult',
        'node_selector': 'V1NodeSelector'
    }

    attribute_map = {
        'allocation_timestamp': 'allocationTimestamp',
        'devices': 'devices',
        'node_selector': 'nodeSelector'
    }

    def __init__(self, allocation_timestamp=None, devices=None, node_selector=None, local_vars_configuration=None):  # noqa: E501
        """V1beta2AllocationResult - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._allocation_timestamp = None
        self._devices = None
        self._node_selector = None
        self.discriminator = None

        if allocation_timestamp is not None:
            self.allocation_timestamp = allocation_timestamp
        if devices is not None:
            self.devices = devices
        if node_selector is not None:
            self.node_selector = node_selector

    @property
    def allocation_timestamp(self):
        """Gets the allocation_timestamp of this V1beta2AllocationResult.  # noqa: E501

        AllocationTimestamp stores the time when the resources were allocated. This field is not guaranteed to be set, in which case that time is unknown.  This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gate.  # noqa: E501

        :return: The allocation_timestamp of this V1beta2AllocationResult.  # noqa: E501
        :rtype: datetime
        """
        return self._allocation_timestamp

    @allocation_timestamp.setter
    def allocation_timestamp(self, allocation_timestamp):
        """Sets the allocation_timestamp of this V1beta2AllocationResult.

        AllocationTimestamp stores the time when the resources were allocated. This field is not guaranteed to be set, in which case that time is unknown.  This is an alpha field and requires enabling the DRADeviceBindingConditions and DRAResourceClaimDeviceStatus feature gate.  # noqa: E501

        :param allocation_timestamp: The allocation_timestamp of this V1beta2AllocationResult.  # noqa: E501
        :type: datetime
        """

        self._allocation_timestamp = allocation_timestamp

    @property
    def devices(self):
        """Gets the devices of this V1beta2AllocationResult.  # noqa: E501


        :return: The devices of this V1beta2AllocationResult.  # noqa: E501
        :rtype: V1beta2DeviceAllocationResult
        """
        return self._devices

    @devices.setter
    def devices(self, devices):
        """Sets the devices of this V1beta2AllocationResult.


        :param devices: The devices of this V1beta2AllocationResult.  # noqa: E501
        :type: V1beta2DeviceAllocationResult
        """

        self._devices = devices

    @property
    def node_selector(self):
        """Gets the node_selector of this V1beta2AllocationResult.  # noqa: E501


        :return: The node_selector of this V1beta2AllocationResult.  # noqa: E501
        :rtype: V1NodeSelector
        """
        return self._node_selector

    @node_selector.setter
    def node_selector(self, node_selector):
        """Sets the node_selector of this V1beta2AllocationResult.


        :param node_selector: The node_selector of this V1beta2AllocationResult.  # noqa: E501
        :type: V1NodeSelector
        """

        self._node_selector = node_selector

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1beta2AllocationResult):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1beta2AllocationResult):
            return True

        return self.to_dict() != other.to_dict()
